@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2003 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """Portable, NFS-safe file locking with timeouts.
 
@@ -546,8 +546,8 @@ def _seed():
     except EnvironmentError, e:
         if e.errno <> errno.ENOENT:
             raise
-        import sha
-        d = sha.new(`os.getpid()`+`time.time()`).hexdigest()
+        from Mailman.Utils import sha_new
+        d = sha_new(`os.getpid()`+`time.time()`).hexdigest()
     random.seed(d)
 
 
