@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2015 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -72,6 +72,8 @@ from Mailman import i18n
 from Mailman.Logging.Syslog import syslog
 
 _ = i18n._
+def D_(s):
+    return s
 
 EMPTYSTRING = ''
 OR = '|'
@@ -260,6 +262,27 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             user = Utils.ObscureEmail(user)
         return '%s/%s' % (url, urllib.quote(user.lower()))
 
+    def GetDescription(self, cset=None, errors='xmlcharrefreplace'):
+        # Get list's description in charset specified by cset.
+        # If cset is None, it uses charset of context language.
+        mcset = Utils.GetCharSet(self.preferred_language)
+        if cset is None:
+            # translation context may not be initialized
+            trns = i18n.get_translation()
+            if trns is None:
+                ccset = 'us-ascii'
+            else:
+                ccset = i18n.get_translation().charset() or 'us-ascii'
+        else:
+            ccset = cset
+        if isinstance(self.description, unicode):
+            return self.description.encode(ccset, errors)
+        if mcset == ccset:
+            return self.description
+        return Utils.xml_to_unicode(self.description, mcset).encode(ccset,
+                                                                    errors)
+
+
 
     #
     # Instance and subcomponent initialization
@@ -634,6 +657,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             if e.errno <> errno.ENOENT: raise
             # The file doesn't exist yet
             return None, e
+        now = int(time.time())
         try:
             try:
                 dict = loadfunc(fp)
@@ -645,8 +669,9 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         finally:
             fp.close()
         # Update the timestamp.  We use current time here rather than mtime
-        # so the test above might succeed the next time.
-        self.__timestamp = int(time.time())
+        # so the test above might succeed the next time.  And we get the time
+        # before unpickling in case it takes more than a second.  (LP: #266464)
+        self.__timestamp = now
         return dict, None
 
     def Load(self, check_version=True):
@@ -782,8 +807,16 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             self.reply_to_address = ''
             self.reply_goes_to_list = 0
         # Legacy topics may have bad regular expressions in their patterns
+        # Also, someone may have broken topics with, e.g., config_list.
         goodtopics = []
-        for name, pattern, desc, emptyflag in self.topics:
+        for value in self.topics:
+            try:
+                name, pattern, desc, emptyflag = value
+            except ValueError:
+                # This value is not a 4-tuple. Just log and drop it.
+                syslog('error', 'Bad topic "%s" for list: %s',
+                       value, self.internal_name())
+                continue
             try:
                 orpattern = OR.join(pattern.splitlines())
                 re.compile(orpattern)
@@ -809,6 +842,8 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         # check for banned address
         pattern = self.GetBannedPattern(invitee)
         if pattern:
+            syslog('vette', '%s banned invitation: %s (matched: %s)',
+                   self.real_name, invitee, pattern)
             raise Errors.MembershipIsBanned, pattern
         # Hack alert!  Squirrel away a flag that only invitations have, so
         # that we can do something slightly different when an invitation
@@ -896,6 +931,19 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             syslog('vette', '%s banned subscription: %s%s (matched: %s)',
                    realname, email, whence, pattern)
             raise Errors.MembershipIsBanned, pattern
+        # See if this is from a spamhaus listed IP.
+        if remote and mm_cfg.BLOCK_SPAMHAUS_LISTED_IP_SUBSCRIBE:
+            if Utils.banned_ip(remote):
+                whence = ' from %s' % remote
+                syslog('vette', '%s banned subscription: %s%s (Spamhaus IP)',
+                       realname, email, whence)
+                raise Errors.MembershipIsBanned, 'Spamhaus IP'
+        # See if this is from a spamhaus listed domain.
+        if email and mm_cfg.BLOCK_SPAMHAUS_LISTED_DBL_SUBSCRIBE:
+            if Utils.banned_domain(email):
+                syslog('vette', '%s banned subscription: %s (Spamhaus DBL)',
+                       realname, email)
+                raise Errors.MembershipIsBanned, 'Spamhaus DBL'
         # Sanity check the digest flag
         if digest and not self.digestable:
             raise Errors.MMCantDigestError
@@ -1011,6 +1059,12 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         # and confirmations.
         pattern = self.GetBannedPattern(email)
         if pattern:
+            if whence:
+                source = ' from %s' % whence
+            else:
+                source = ''
+            syslog('vette', '%s banned subscription: %s%s (matched: %s)',
+                   self.real_name, email, source, pattern)
             raise Errors.MembershipIsBanned, pattern
         # Do the actual addition
         self.addNewMember(email, realname=name, digest=digest,
@@ -1027,13 +1081,20 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         syslog('subscribe', '%s: new%s %s, %s', self.internal_name(),
                kind, formataddr((name, email)), whence)
         if ack:
-            self.SendSubscribeAck(email, self.getMemberPassword(email),
-                                  digest, text)
+            lang = self.preferred_language
+            otrans = i18n.get_translation()
+            i18n.set_language(lang)
+            try:
+                self.SendSubscribeAck(email, self.getMemberPassword(email),
+                                      digest, text)
+            finally:
+                i18n.set_translation(otrans)
         if admin_notif:
             lang = self.preferred_language
             otrans = i18n.get_translation()
             i18n.set_language(lang)
             try:
+                whence = "" if whence is None else "(" + _(whence) + ")"
                 realname = self.real_name
                 subject = _('%(realname)s subscription notification')
             finally:
@@ -1044,6 +1105,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
                 "adminsubscribeack.txt",
                 {"listname" : realname,
                  "member"   : formataddr((name, email)),
+                 "whence"   : whence
                  }, mlist=self)
             msg = Message.OwnerNotification(self, subject, text)
             msg.send(self)
@@ -1080,6 +1142,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
                 'adminunsubscribeack.txt',
                 {'member'  : name,
                  'listname': self.real_name,
+                 "whence"   : "" if whence is None else "(" + _(whence) + ")"
                  }, mlist=self)
             msg = Message.OwnerNotification(self, subject, text)
             msg.send(self)
@@ -1175,6 +1238,9 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         # exception.
         pattern = self.GetBannedPattern(newaddr)
         if pattern:
+            syslog('vette',
+                   '%s banned address change: %s -> %s (matched: %s)',
+                   self.real_name, oldaddr, newaddr, pattern)
             raise Errors.MembershipIsBanned, pattern
         # It's possible they were a member of this list, but choose to change
         # their membership globally.  In that case, we simply remove the old
@@ -1256,6 +1322,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
     # Confirmation processing
     #
     def ProcessConfirmation(self, cookie, context=None):
+        global _
         rec = self.pend_confirm(cookie)
         if rec is None:
             raise Errors.MMBadConfirmation, 'No cookie record for %s' % cookie
@@ -1265,7 +1332,8 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         except ValueError:
             raise Errors.MMBadConfirmation, 'op-less data %s' % (rec,)
         if op == Pending.SUBSCRIPTION:
-            whence = 'via email confirmation'
+            _ = D_
+            whence = _('via email confirmation')
             try:
                 userdesc = data[0]
                 # If confirmation comes from the web, context should be a
@@ -1274,7 +1342,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
                 # context is a Message and isn't relevant, so ignore it.
                 if isinstance(context, UserDesc):
                     userdesc += context
-                    whence = 'via web confirmation'
+                    whence = _('via web confirmation')
                 addr = userdesc.address
                 fullname = userdesc.fullname
                 password = userdesc.password
@@ -1282,6 +1350,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
                 lang = userdesc.language
             except ValueError:
                 raise Errors.MMBadConfirmation, 'bad subscr data %s' % (data,)
+            _ = i18n._
             # Hack alert!  Was this a confirmation of an invitation?
             invitation = getattr(userdesc, 'invitation', False)
             # We check for both 2 (approval required) and 3 (confirm +
@@ -1304,11 +1373,14 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             return op, addr, password, digest, lang
         elif op == Pending.UNSUBSCRIPTION:
             addr = data[0]
-            # Log file messages don't need to be i18n'd
+            # Log file messages don't need to be i18n'd, but this is now in a
+            # notice.
+            _ = D_
             if isinstance(context, Message.Message):
-                whence = 'email confirmation'
+                whence = _('email confirmation')
             else:
-                whence = 'web confirmation'
+                whence = _('web confirmation')
+            _ = i18n._
             # Can raise NotAMemberError if they unsub'd via other means
             self.ApprovedDeleteMember(addr, whence=whence)
             return op, addr
